<?php
/*
* Project:     XML for MagpieRSS: a simple RSS integration tool
* File:        xml_fetch.inc, a simple functional interface
to fetching and parsing XML
* Author:      Bram Brambring  <bram@brambring.nl>
* License:     GPL
* Script requires an installation of MagpieRSS
* Tested with magpie 1.72
*
*/
defined('_JEXEC') or die('Restricted access');
if (!defined('DIR_SEP')) {
    define('DIR_SEP', DIRECTORY_SEPARATOR);
}

if (!defined('MAGPIE_DIR')) {
    define('MAGPIE_DIR', dirname(__FILE__) . DIR_SEP);
}
if (!defined('MAGPIE_PREFETCHTIME')) {
    define('MAGPIE_PREFETCHTIME', 3600);
}


require_once(MAGPIE_DIR . 'util.php');

if (! function_exists('storeitem')) {
    function storeitem(&$item)
    {
        return;
    }
}


/*=======================================================================*\
Function: xml_fetch:
Purpose:  returns  object for the give url
maintain the cache
Input:    url of RSS file
XML class
cache age
Output:   parsed RSS object (see rss_parse.inc)
NOTES:
see rss_fetch.inc
\*=======================================================================*/
if (!function_exists('sys_get_temp_dir')) {
    function sys_get_temp_dir()
    {
        if ($temp=getenv('TMP')) {
            return $temp;
        }
        if ($temp=getenv('TEMP')) {
            return $temp;
        }
        if ($temp=getenv('TMPDIR')) {
            return $temp;
        }
        $temp=tempnam(__FILE__, '');
        if (file_exists($temp)) {
            unlink($temp);
            return dirname($temp);
        }
        return null;
    }
}

function noop(&$item)
{
    #nothing;
}
function bare_fetch(
    $url,
    $class,
    $input_encoding = false,
    $callback = 'noop'
) {

    init();
    if (! function_exists($callback)) {
        $callback="noop";
    }
    if (!isset($url) || $url=='') {
        $rss = new stdClass();
        $rss->ERROR="fetch_rss called without a url";
        $rss->status=false;
        trigger_error($rss->ERROR);
        return $rss;
    }
    $rss = new $class($url, $input_encoding, $callback, false);
    if ($rss && count($rss->items??0) > 0) {
        $rss->status=true;
        $rss->examples=array_slice($rss->items, 0, 4);
    } else {
        if (!is_object($rss)) {
            $rss=new StdClass();
        }
        $rss->status=false;
        $errormsg = "Failed to parse RSS file.";
        if ($rss->ERROR) {
            $errormsg .= " (" . $rss->ERROR . ")";
        }
        $rss->ERROR=$errormsg;
        $rss->error($rss->ERROR);
        return $rss;
    }
    unset($rss->items);

    # 'fall thru'

    return $rss;
}

function get_cache_dir()
{
    $curdir=dirname(__FILE__);
    #php seems t turn the slashes right use unix style fix when broken
    if (defined('FORCE_TEMP_DIR')) {
        $tmppaths[]=sys_get_temp_dir();
    } else {
        if (defined('USE_TEMP_DIR')) {
            $tmppaths[]=sys_get_temp_dir();
        }
        $tmppaths[]=$curdir.'/../../../cache';
        $tmppaths[]=$curdir.'/../../../../cache';
        $tmppaths[]=$curdir.'/../../cache';
        $tmppaths[]=$curdir.'/../cache';
        $tmppaths[]=$curdir.'/cache';
    }
    foreach ($tmppaths as $d) {
        if (is_writable($d) || (file_exists($d.'/com_datafeeds')&& is_writable($d.'/com_datafeeds'))) {
            $cache_dir=$d.'/com_datafeeds/'; # joomla style
            if (! file_exists($cache_dir)) {
                mkdir($cache_dir);
                chmod($cache_dir, 0777);
            }
            break;
        }
    }
    if (empty($cache_dir)) {
        print("We need a cache dir for curl/prefetching, please create administrator/cache and/or make it writable to the webserver\n");
        exit;
    #return false;
    } else {
        return realpath($cache_dir).'/';
    }
}
function get_cache_file($url)
{
    global $feed_opts;
    $feed=preg_replace('/[^a-z0-9]/', '_', strtolower($feed_opts['feed']));
    $dir=get_cache_dir();
    return $dir ? $dir.$feed.'-'.crc32($url) : false;
}
function fetch_curl($url,$headers=[])
{
    $cache_file=get_cache_file($url);
    if ($cache_file !== false) {
        if (true || (! file_exists($cache_file))|| (filesize($cache_file) < 10 )  || (filemtime($cache_file) < time()-MAGPIE_PREFETCHTIME)) { # todo make define
            print "Using curl to preload $cache_file\n";
            $ch = curl_init($url);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
            if (ini_get('open_basedir') == null) {
                curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
                curl_setopt($ch, CURLOPT_MAXREDIRS, 2);
            }
            if (! defined('DISABLE_GZIP_ENCODING')) {
                curl_setopt($ch, CURLOPT_ENCODING, '');
            }
            if (is_file($cache_file)) {
                unlink($cache_file);
            }
            $cf=fopen($cache_file, "wb");
            if (! $cf) {
                return false;
            }
            curl_setopt($ch, CURLOPT_FILE, $cf);
            $res=curl_exec($ch);
            if ($res === false) {
                echo $url . "\n";
                echo curl_error($ch)."\n";
                unlink($cache_file);
            }
            curl_close($ch);
            fflush($cf);
            fclose($cf);
            chmod($cache_file, 0666);
        } else {
            print "Using cache: $cache_file ".filesize($cache_file)."\n";
        }
    }
    return $cache_file;
}

function fetch_fopen($url)
{
    $cache_file=get_cache_file($url);
    if ($cache_file !== false) {
        if ((! file_exists($cache_file))||(filesize($cache_file) < 10 ) || (filemtime($cache_file) < time()-MAGPIE_PREFETCHTIME)) { # todo make define
            print "Using force fetch open $cache_file\n";
            $in=fopen($url, 'rb');
            $out=fopen($cache_file, 'wb');
            if ($in && $out) {
                while (!feof($in)) {
                    $buffer = fread($in, 4096);
                    fwrite($out, $buffer);
                }
                fclose($in);
                fclose($out);
            }
        } else {
            print "Using cache: $cache_file ".filesize($cache_file)."\n";
        }
    }
    return $cache_file;
}

function getBolAuthHeader() {
	 if (! defined('BOLCLIENTID') || ! defined('BOLCLIENTSECRET')) {
		 $msg="Bol.com API requires Client ID and Secret. More on <a target=\"_blank\" href=\"https://affiliatefeeds.nl/nl/238-bol-com-product-api.html\">Affiliatefeeds.nl</a>\n";
		 print $msg;
	$this->error($msg);
			                        $this->status=false;
			                        $this->items=array();
						                           return;
	 }
        $url='https://login.bol.com/token?grant_type=client_credentials';
	 $encoded=base64_encode(BOLCLIENTID.':'.BOLCLIENTSECRET);
    	$basic=['Authorization: Basic '.$encoded];
	$cache_file=get_cache_dir().'bol_auth-'.crc32($encoded);
        if ((! file_exists($cache_file))|| (filesize($cache_file) < 10 )  || (filemtime($cache_file) < time()-60)) { 
	print "Bol Authorization: Basic Fetched\n";
	$curl = curl_init($url);
	curl_setopt($curl, CURLOPT_URL, $url);
	curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
	curl_setopt($curl, CURLOPT_POST, 1);
	curl_setopt($curl, CURLOPT_HTTPHEADER, $basic);
	$resp = curl_exec($curl);
	file_put_contents($cache_file,$resp);
	curl_close($curl);

	} else {
		$resp=file_get_contents($cache_file);
	}
	$json=json_decode($resp);
    	return ['Authorization: Bearer '.$json->access_token];

}
function prefetchBol($url) {
    $headers=getBolAuthHeader();
    $url=fetch_curl($url,$headers);
    print "$url";
    return $url;
}

function prefetch($url)
{
    if (preg_match('#^(https|http|ftp)://#i', $url)) {
        if (ini_get('allow_url_fopen') != 1) {
            $url=fetch_curl($url);
        } elseif (defined('FORCE_CURL')) {
            $url=fetch_curl($url);
        } elseif (defined('FORCE_PREFETCH')) {
            $url=fetch_fopen($url);
        }
    }
    return $url;
}




class TradetrackerRSS extends MagpieRSS
{
    public $current_categories   = [];  // item currently being parsed
    public $current_fields   = [];  // item currently being parsed
    public $example_raw   = [];
    public $incategories       = false;
    public $incategory     = false;
    public $inadditonal        = false;
    public $infield        = false;
    public $csv                = false;
    public $count              = 0;
    public $parser;
    public $error = false;
    public $callback='noop';

    protected function _prefetch($url)
    {

        if (strpos($url, '..') === 0) {
            $url=dirname(__FILE__).'/'.$url ; # php will fix / and \
        }
        $newurls=[];
        $urls=explode(" ", $url); #multi url for one configuration hardly used
        foreach ($urls as $url) {
            if (strlen($url) < 5) {
                continue;
            }
            $url=prefetch($url);
            if ($url !== false) {
                $newurls[]=$url;
            }
        }
        return join(' ', $newurls);
    }

    public function __construct(
        $url,
        $input_encoding = false,
        $callback = 'noop',
        $keep_items = true
    ) {
        # if PHP xml isn't compiled in, die
#
        if (!function_exists('xml_parser_create')) {
            $this->error(
                "Failed to load PHP's XML Extension. " .
                "http://www.php.net/manual/en/ref.xml.php",
                E_USER_ERROR
            );
            $this->status=false;
            $this->items=array();
            return;
        }
        $this->callback=$callback;
        $this->keep_items=$keep_items;
        $this->count=0;
        $url=$this->_prefetch($url);

        if (!$url) {
            $this->status=false;
            $this->items=array();
            $this->error('Creation of cache directory or file failed. Ensure administrator/cache/* is writable', E_USER_ERROR);
            $this->url=false;
            return;
        }
        
        $urls=explode(" ", $url); #multi url for one configuration
        foreach ($urls as $url) {
            if (strlen($url) < 5) {
                continue;
            }


            $this->parser = $this->create_parser();



            # pass in parser, and a reference to this object
            # setup handlers
#
            xml_set_object($this->parser, $this);
            xml_set_element_handler(
                $this->parser,
                'feed_start_element',
                'feed_end_element'
            );

            xml_set_character_data_handler($this->parser, 'feed_cdata');
            if (defined('USE_GZOPEN')) {
                $file_function='gzopen';
            } else {
                $file_function='fopen';
            }

            if (!($fp = $file_function($url, 'rb'))) {
                $this->error("Cannot open {$url}", E_USER_ERROR);
                return;
            }

            while (($data = fread($fp, 8192))) {
                if (function_exists($input_encoding)) {
                    $input_encoding($data);
                }
                if (!xml_parse($this->parser, $data, feof($fp))) {
                    $this->error=sprintf(
                        "XML error in %s at line %d column %d</p>\n",
                        $url,
                        xml_get_current_line_number($this->parser),
                        xml_get_current_column_number($this->parser)
                    );
                    print $this->error;
                    #unset ($this->items);

#return;
                }
            }


            $this->destroy_parser($this->parser);
        }
        return;
    }

    public function feed_start_element($p, $element, $attrs)
    {
        $el = $element = strtolower($element);
        $attrs = array_change_key_case($attrs, CASE_LOWER);

        // check for a namespace, and explode if found
        #$ns = false;
        #if ( strpos( $element, ':' ) ) {
        #list($ns, $el) = explode( ':', $element, 2);
        #}
        #if ( $ns and $ns != 'rdf' ) {
        #$this->current_namespace = $ns;
        #}

        # if feed type isn't set, then this is first element of feed
        # identify feed from root element
#

        if ($el == 'products') {
        } elseif ($el == 'fields') {
        } elseif ($el == 'product') {
            $this->initem = true;
        } elseif ($el == 'categories') {
            $this->incategories = true;
        } elseif ($el == 'category') {
            $this->incategory = true;
            if (isset($attrs['name'])) {
                $this->concat_field($this->current_categories[strtolower($attrs['name'])], $attrs['path']);
                $this->concat_field($this->current_item[strtolower($attrs['name'])], $attrs['path']);
            } else {
                $this->concat_field($this->current_categories[], $attrs['path']);
                $this->concat_field($this->current_item['category_path'], $attrs['path']);
            }
        } elseif ($el == 'additional') {
            $this->inadditional = true;
        } elseif ($el == 'field') {
            if (isset($attrs['name'])) {
                #$name=strtolower(preg_replace("/[0-9]*$/","",$attrs['name']));
                $name=strtolower($attrs['name']);
                if (isset($attrs['value'])) {
                    $this->concat_field($this->current_item["f_".$name], $attrs['value']);
                } else {
                    $this->infield = true;
                    array_unshift($this->stack, $name);
                }
            }
        } // set stack[0] to current element
        else {
            array_unshift($this->stack, $el);
        }
    }

    public function feed_end_element($p, $el)
    {
        $el = strtolower($el);

        if ($el == 'product') {
            $cat=explode("/", join("/", $this->current_categories));
            foreach ($cat as $k => $v) {
                $this->current_item["c_$k"]=$v;
            }

            $this->current_item['categories']=$cat;
            $this->store_item($this->current_item);
            $this->current_categories = array();
            #$this->current_fields = array();
            $this->initem = false;
        } elseif ($el == 'additional') {
            $this->inadditional = false;
        } elseif ($el == 'field') {
            if ($this->infield) {
                $this->infield = false;
                array_shift($this->stack);
            }
        } elseif ($el == 'categories') {
            $this->incategories = false;
        } elseif ($el == 'category') {
            $this->incategory = false;
        } elseif ($el == 'products') {
        } elseif ($el == 'fields') {
        } else {
            array_shift($this->stack);
        }

        #$this->current_namespace = false;
    }
    public function store_item(&$item)
    {
        #duur maar we hebben voorbeeld nodig,
        #niet alle items hebben alle velden
        if (is_array($item)) {
            $this->example_raw=array_merge($this->example_raw, $item);
        }
        $item['count']=$this->count++;
        $func= $this->callback;
        $func($item);
        storeitem($item);

        if (($this->keep_items) || ($this->count <4)) {
            $this->items[] = $item;
        }
        $item = array();
    }
    public function concat_field(&$str1, $str2 = "")
    {
        if (!isset($str1)) {
            $str1=$str2;
        } else {
            $str1 .= ", " . $str2;
        }
    }



    #simpel cdata for aff feeds
    public function feed_cdata($p, $text)
    {
        $current_el = join('_', array_reverse($this->stack));
        #todo ? performance als de append hier komt ipv functie aanroep.
        $this->append($current_el, $text);
    }
    #simpel append for aff feeds
    public function append($el, $text)
    {
        if (!$el) {
            return;
        }
        if ($this->initem) {
            $this->concat(
                $this->current_item[ $el ],
                $text
            );
        }
    }
} // end class RSS




class AffiliateRSS extends TradetrackerRSS
{
    public function feed_start_element($p, $element, $attrs)
    {
        $el = $element = strtolower($element);
        $attrs = array_change_key_case($attrs, CASE_LOWER);


        if (($el == 'items') || ($el == 'one-trip-per-accommodation-flat')) {
        } elseif (($el == 'item')||($el=='trip')) {
            $this->initem = true;
        } // set stack[0] to current element
        else {
            array_unshift($this->stack, $el);
        }
    }




    public function feed_end_element($p, $el)
    {
        $el = strtolower($el);

        if (($el == 'item')||($el=='trip')) {
            $this->store_item($this->current_item);
            $this->current_categories = array();
            $this->current_fields = array();
            $this->initem = false;
        } elseif (($el == 'items') || ($el == 'one-trip-per-accommodation-flat')) {
        } else {
            array_shift($this->stack);
        }

        #$this->current_namespace = false;
    }
} // end class RSS

class DaisyconRSS extends TradetrackerRSS
{
    public function feed_start_element($p, $element, $attrs)
    {
        $el = $element = strtolower($element);
        $attrs = array_change_key_case($attrs, CASE_LOWER);


        if ($el == 'product') {
            $this->initem = true;
            $this->stack=array();
        } elseif ($el == 'product_info') {
        } elseif ($el == 'images') {
            #} elseif  ($el == 'image' ) {
#$this->inimage = true;
#array_unshift($this->stack, $el);

// set stack[0] to current element
        } else {
            array_unshift($this->stack, $el);
        }
    }




    public function feed_end_element($p, $el)
    {
        $el = strtolower($el);

        if ($el == 'product') {
            $this->store_item($this->current_item);
            $this->current_categories = array();
            $this->current_fields = array();
            $this->initem = false;
        } elseif ($el == 'product_info') {
        } elseif ($el == 'update_info') {
            unset($this->current_item['update_info']);
            array_shift($this->stack);
        } elseif ($el == 'images') {
        } elseif ($el == 'image') {
            if ($this->current_item['image_tag']=='default') {
                $this->current_item['image']=$this->current_item['image_location'];
            }
            $this->current_item['image_'.str_replace(' ', '_', $this->current_item['image_tag'])]=$this->current_item['image_location'];
            $this->current_item['image_'.str_replace(' ', '_', $this->current_item['image_size'])]=$this->current_item['image_location'];
            unset($this->current_item['image_tag']);
            unset($this->current_item['image_location']);
            unset($this->current_item['image_size']);
            unset($this->current_item['image_type']);
            array_shift($this->stack);
        } else {
            array_shift($this->stack);
        }

        #$this->current_namespace = false;
    }
} // end class RSS


#CSV feeds
class AutoCSV extends TradetrackerRSS
{
    protected $encode=false;
    protected $delimiter=false;
    protected $skip=false;
    protected $header=true;
    protected $enclosure='"';
    protected $header_array=[];
    protected $escape ='\\"';


    public function __construct(
        $url,
        $input_encoding = false,
        $callback = 'noop',
        $keep_items = true
    ) {
        $this->callback=$callback;
        $this->url=$this->_prefetch($url);
        $this->keep_items=$keep_items;

        if (function_exists($input_encoding)) {
            $this->encode=$input_encoding;
        } elseif (strtoupper($input_encoding) == 'ISO-8859-1') {
            $this->encode='utf8_encode';
        }

        $this->_parse();
    }

    protected function _parse()
    {
        $length = 0;
        $this->count=0;

        $first = false;
        if (defined('USE_GZOPEN')) {
            $file_function='gzopen';
        } else {
            $file_function='fopen';
        }
        if (!($handle = $file_function($this->url, 'r'))) {
            $this->error("Cannot open {$this->url}", E_USER_ERROR);
            return;
        }
        if ($this->delimiter === false) {
            $count=-1;
            $header=fgets($handle);
            foreach (array(';','|',',',"\t") as $v) {
                $c=substr_count($header, $v);
                if ($c > $count) {
                    $this->delimiter=$v;
                    $count=$c;
                }
            }
            fseek($handle, 0);
        }
        if ($this->skip) {
            for ($i=0; $i< $this->skip; $i++) {
                $data = fgetcsv($handle, $length, $this->delimiter, $this->enclosure);
            }
        }
        if ($this->header) {
            $data = fgetcsv($handle, $length, $this->delimiter, $this->enclosure);
            if (is_array($data)) {
                foreach ($data as $k => $v) {
                    if (! $v) {
                        $v="Veld $k";
                    }
                    $this->header_array[$k]=strtolower($v);
                }
            } else {
                print "Download of datafeed failed, please check feed url and write permissions on cache";
            }
        }
        $this->header_count=count($this->header_array);
        while (($data = fgetcsv($handle, $length, $this->delimiter, $this->enclosure)) !== false) {
            if ($this->header_count) {
                $data=$this->_arraycombine($data);
            }
            if ($this->encode) {
                $data=array_map($this->encode, $data);
            }
            $this->store_item($data);
            $data=array();
        }
        fclose($handle);
    }



    public function _arraycombine(&$a2)
    {
        $ra=array();
        for ($i=0; $i< $this->header_count; $i++) {
            if (isset($a2[$i])) {
                $ra[$this->header_array[$i]] = $a2[$i];
            }
        }
        return $ra;
    }

    public function _array_combine(&$a1, &$a2) # legacy maybe there are custom parsers using it.
    {
        $ra=array();
        $c=count($a1);
        for ($i=0; $i< $c; $i++) {
            if ($a2[$i]) {
                $ra[$a1[$i]] = $a2[$i];
            }
        }
        return $ra;
    }
}

class AffiliateCSV extends AutoCSV
{
    public function __construct($url, $input_encoding = false, $callback = 'noop', $keep_items = true)
    {
        $this->delimiter =";";
        parent::__construct($url, $input_encoding, $callback, $keep_items);
    }
}
class M4NCSV extends AutoCSV
{
    public function __construct(
        $url,
        $input_encoding = false,
        $callback = 'noop',
        $keep_items = true
    ) {
        $this->delimiter ="\t";
        parent::__construct($url, $input_encoding, $callback, $keep_items);
    }
    #obsolete remains for backward comp.
}

class WebgainsCSV extends AutoCSV
{
    public function __construct(
        $url,
        $input_encoding = false,
        $callback = 'noop',
        $keep_items = true
    ) {
        $this->delimiter =",";
        parent::__construct($url, $input_encoding, $callback, $keep_items);
    }
}
class TradedoublerCSV extends AutoCSV
{
    public function __construct(
        $url,
        $input_encoding = false,
        $callback = 'noop',
        $keep_items = true
    ) {
        $this->delimiter ="|";
        parent::__construct($url, $input_encoding, $callback, $keep_items);
    }
}




class TradedoublerRSS extends TradetrackerRSS
{
    public $current_categories   = array();  // item currently being parsed
    public $current_fields   = array();  // item currently being parsed
    public $incategories       = false;
    public $incategory     = false;
    public $inadditonal        = false;
    public $infield        = false;



    public function feed_start_element($p, $element, $attrs)
    {
        $el = $element = strtolower($element);
        $attrs = array_change_key_case($attrs, CASE_LOWER);


        if ($el == 'products') {
        } elseif ($el == 'fields') {
        } elseif ($el == 'product') {
            $this->initem = true;
        } elseif ($el == 'tdcategories') {
            $this->incategories = true;
        } elseif ($el == 'category') {
            $this->incategory = true;
            $this->concat_field($this->current_item['_category'], $attrs['path']??'');
            $this->concat_field(strtolower($this->current_categories[$attrs['name']]), $attrs['path']??'');
        } elseif ($el == 'additional') {
            $this->inadditional = true;
        } elseif ($el == 'field') {
            $this->infield = true;
            array_unshift($this->stack, $el);
        } // set stack[0] to current element
        else {
            array_unshift($this->stack, $el);
        }
    }

    public function feed_end_element($p, $el)
    {
        $el = strtolower($el);

        if ($el == 'product') {
            #$this->current_item['categories']=$this->current_categories;
            unset($this->current_item['fields']);
            unset($this->current_item['tdcategory']);
            $this->store_item($this->current_item);
            $this->current_categories = array();
            $this->initem = false;
        } elseif ($el == 'additional') {
            $this->inadditional = false;
        } elseif ($el == 'field') {
            $this->infield = false;
            $name=$this->field['name'];
            $value=$this->field['value'];
            unset($this->field);
            $this->current_item['f_'.strtolower($name)]=$value;
            array_shift($this->stack);
        } elseif ($el == 'tdcategories') {
            $this->incategories = false;
        } elseif ($el == 'category') {
            $this->incategory = false;
        } elseif ($el == 'fields') {
        } elseif ($el == 'products') {
        } elseif (($el == 'name' || $el =='value') &&  $this->infield) {
            $current_el = join('_', array_reverse($this->stack));
            $this->field[$el]=trim($this->current_item[$current_el]);
            unset($this->current_item[$current_el]);
            array_shift($this->stack);
        } else {
            array_shift($this->stack);
        }

        #$this->current_namespace = false;
    }
} // end class RSS



class WebgainsRSS extends TradedoublerRSS
{
    #tradedoubler class does most of the job.
    #let's explode the path of the _category ( is > )
    public function feed_start_element($p, $el, $attrs)
    {
        parent::feed_start_element($p, $el, $attrs);
        $el = strtolower($el);
        if ($el == 'categories') {
            if (!empty($this->current_item[ $el ])) {
                $this->current_item[ $el ]  .=",";
            }
        }
    }
    public function feed_end_element($p, $el)
    {
        $el = strtolower($el);

        if ($el == 'product') {
            unset($this->current_item['fields']);
            $this->store_item($this->current_item);
            $this->current_categories = array();
            $this->initem = false;
        } elseif ($el == 'additional') {
            $this->inadditional = false;
        } elseif ($el == 'field') {
            $this->infield = false;
            $name=$this->field['name'];
            $value=$this->field['value'];
            unset($this->field);
            $this->current_item['f_'.strtolower($name)]=$value;
        } elseif ($el == 'category') {
            $cat=explode('>', $this->current_item['_category']);
            foreach ($cat as $k => $v) {
                $this->current_item["category_path_$k"]=trim($v, ",");
            }
            $this->incategory = false;
        } elseif ($el == 'products') {
        } elseif (($el == 'name' || $el =='value') &&  $this->infield) {
            $current_el = join('_', array_reverse($this->stack));
            $this->field[$el]=trim($this->current_item[$current_el]);
            unset($this->current_item[$current_el]);
            array_shift($this->stack);
        } else {
            array_shift($this->stack);
        }

        #$this->current_namespace = false;
    }
}
class ProductserveRSS extends TradetrackerRSS
{
    public function feed_start_element($p, $element, $attrs)
    {
        $el = $element = strtolower($element);
        $attrs = array_change_key_case($attrs, CASE_LOWER);


        # if feed type isn't set, then this is first element of feed
        # identify feed from root element
#

        if ($el == 'merchantproductfeed') {
        } elseif ($el == 'merchant') {
            $this->current_item['merchant_name']=$attrs['name']??'';
            $this->current_item['merchant_id']=$attrs['id']??'';
        } elseif ($el == 'text') {
        } elseif ($el == 'uri') {
        } elseif ($el == 'cat') {
        } elseif ($el == 'prod') {
            $this->initem = true;
            $this->current_item['prod_id']=$attrs['id']??'';
        } else {
            array_unshift($this->stack, $el);
        }
    }

    public function feed_end_element($p, $el)
    {
        $el = strtolower($el);

        if ($el == 'prod') {
            if (empty($this->current_item['description'])) {
                $this->current_item['description']=$this->current_item['desc']??'';
                unset($this->current_item['desc']);
            }
            $this->store_item($this->current_item);
            $this->initem = false;
        } elseif ($el == 'merchant') {
        } elseif ($el == 'merchantproductfeed') {
        } else {
            array_shift($this->stack);
        }

        #$this->current_namespace = false;
    }
}
class NetaffiliationRSS extends TradetrackerRSS
{
    public function feed_end_element($p, $el)
    {
        $el = strtolower($el);

        if ($el == 'product') {
            $cat=explode(">", $this->current_item['rubrique']);
            foreach ($cat as $k => $v) {
                $this->current_item["rubrique_$k"]=$v;
            }
            if (empty($this->current_item['description'])) {
                $this->current_item['description']=$this->current_item['descriptif']??'';
                unset($this->current_item['descriptif']);
            }

            $this->store_item($this->current_item);
            $this->initem = false;
        } elseif ($el == 'products') {
        } else {
            array_shift($this->stack);
        }

        #$this->current_namespace = false;
    }
}


###############################################################################
#
# Basic example parser
#
# https://affiliatefeeds.nl/en/46-general/3086-creating-a-parser.html
#
###############################################################################


class BasicRSS extends TradetrackerRSS
{
    public $item_tag='item';
    public $multitag=array();
    public $description='';

    public function feed_start_element($p, $el, $attrs)
    {
        $el =  strtolower($el);
        if ($el == $this->item_tag) {
            $this->initem = true;
        } else {
            if ($this->initem) {
                if ($el == $this->description) {
                    $el='description';
                }
                if (isset($this->multitag[$el])) {
                    $a=$this->multitag[$el]++;
                    $el .='_'.$a;
                } else {
                    $this->multitag[$el]=0;
                }
                array_unshift($this->stack, $el);
            }
        }
    }

    public function feed_end_element($p, $el)
    {
        $el = strtolower($el);

        if ($el == $this->item_tag) {
            $this->store_item($this->current_item);
            $this->initem = false;
            $this->multitag=array();
        } else {
            if ($this->initem) {
                array_shift($this->stack);
            }
        }
    }
}
class GSRSS extends BasicRSS
{
    public $item_tag='entry';
}



class TradetrackerV2 extends TradetrackerRSS
{
    public $inproperties       = false;
    public $invariations       = 0;
    public $inproperty         = false;
    public $hadvalue         = false;
    public $images         = 0;

    public function feed_start_element($p, $element, $attrs)
    {
        $el = $element = strtolower($element);
        $attrs = array_change_key_case($attrs, CASE_LOWER);

        if ($el == 'products') {
        } elseif ($el == 'properties') {
            $this->inproperties=true;
        } elseif ($el == 'variations') {
            $this->invariations=0;
        } elseif ($el == 'variation') {
            $this->invariations++;
        } elseif ($el == 'images') {
            $this->inimages=0;
        } elseif ($el == 'image') {
            $this->inimages++;
            array_unshift($this->stack, 'image_'.$this->inimages);
        } elseif ($el == 'property') {
            $this->inproperty=true;
            $this->initem=false;
            $a=strtolower($attrs['name']).($this->invariations ? '_'.$this->invariations : '');
            if ($a=='url') {
                $a='extra_url' ;
            }
            if (strpos($a, 'image_') === 0) {
                $this->inimages++;
                $a= 'image_'.$this->inimages;
            }
            array_unshift($this->stack, $a);
        } elseif ($el == 'product') {
            $this->current_item['product_id']=$attrs['id']??'';
            $this->initem = true;
        } elseif ($el == 'value' && $this->inproperty) {
            $this->initem=true;
            if ($this->hadvalue) {
                $current_el = join('_', array_reverse($this->stack));
                #todo ? performance als de append hier komt ipv functie aanroep.
                $this->append($current_el, ',');
            }
            $this->hadvalue=true;
        } elseif ($el == 'categories') {
            $this->incategories = true;
            $this->inicats=0;
        } elseif ($el == 'category') {
            $this->inicats++;
            $this->incategory = true;
            $a='';
            if (!empty($attrs['name'])) {
                $a=$attrs['name']??'';
                $this->concat_field($this->current_categories[strtolower($attrs['name'])], $attrs['path']);
                $this->concat_field($this->current_item[strtolower($attrs['name'])], $attrs['path']);
            } else {
                if (!empty($attrs['path'])) {
                    $b=$attrs['path'];
                    $this->concat_field($this->current_categories[], $b);
                    $this->concat_field($this->current_item['category_path'], $b);
                    $c=explode('/', $b, 2);
                    if (count($c)==2 && !empty($c[0])) {
                        $a='category_'.strtolower($c[0]);
                    }
                }
            }
            if (!$a) {
                $a='category_'.$this->inicats;
            }
            array_unshift($this->stack, $a);
        } else {
            array_unshift($this->stack, $el);
        }
    }
    public function feed_end_element($p, $el)
    {
        $el = strtolower($el);

        if ($el == 'product') {
            #$cat=explode("/", join("/",$this->current_categories));
            #foreach ($cat as $k => $v) {
            #$this->current_item["c_$k"]=$v;
            #}

            #$this->current_item['categories']=$cat;
            $this->store_item($this->current_item);
            $this->current_categories = array();
            #$this->current_fields = array();
            $this->initem = false;
        } elseif ($el == 'categories') {
            $this->incategories = false;
        } elseif ($el == 'category') {
            $this->incategory = false;
            array_shift($this->stack);
        } elseif ($el == 'value' && $this->inproperty) {
            $current_el = join('_', array_reverse($this->stack));
            $this->initem=false;
        } elseif ($el == 'property') {
            $this->inproperty=false;
            $this->hadvalue=false;
            $this->initem=true;
            array_shift($this->stack);
        } elseif ($el == 'products') {
        } elseif ($el == 'variations') {
            $this->invariations=0;
        } elseif ($el == 'variation') {
        } elseif ($el == 'images') {
        } elseif ($el == 'properties') {
            $this->inproperties=false;
        } else {
            array_shift($this->stack);
        }

        #$this->current_namespace = false;
    }
}

class JsonCSV extends AutoCSV
{
    public function __construct($url, $input_encoding = false, $callback = 'noop', $keep_items = true)
    {
        $this->callback=$callback;
        $this->url=$this->_prefetch($url);
        $this->keep_items=$keep_items;
        $this->_parse();
    }
    protected function _parse()
    {
        $this->count=0;
        $json=file_get_contents($this->url);
        if ($json) {
            $data=json_decode($json);
            if ($data) {
                foreach ($data as $result) {
                    $result=$this->_squash($result);
                    $result['description']=preg_replace("/\r|\n/ms", "", $result['publicdescription']);
                    $this->store_item($result);
                }
            }
        }
    }

    protected function _squash($array, $prefix = '')
    {
        $flat = array();
        $sep = ".";

        if (!is_array($array)) {
            $array = (array)$array;
        }

        foreach ($array as $key => $value) {
            $_key = strtolower(ltrim($prefix.$sep.$key, "."));

            if (is_array($value) || is_object($value)) {
                // Iterate this one too
                $flat = array_merge($flat, $this->_squash($value, $_key));
            } else {
                $flat[$_key] = $value;
            }
        }

        return $flat;
    }
}
if (! defined('BOLLIMIT')) {
    define('BOLLIMIT', 500);
}
if (! defined('BOLSTEP')) {
    define('BOLSTEP', 100);
}
class BolUrl extends JsonCsv
{
     private $hadvalue         = false;
    public function __construct($url, $input_encoding = false, $callback = 'noop', $keep_items = true)
    {
        $this->callback=$callback;
        $this->url=$url;
        $this->keep_items=$keep_items;
        $this->_parse();
    }

    protected function _parse()
    {
        foreach (explode(' ', trim($this->url)) as $url) {
            $url=trim($url);
            if (!$url) {
                //double spaces
                continue;
            }
            $start=0;
            $apiurl=false;
            $apivars=[];
            $ids=[];
            $search='';
            $parsed=parse_url($url);

            if (isset($parsed['query'])) {
                parse_str($parsed['query'], $vars);
            } else {
                $vars=[];
            }
            $step=min($vars['limit']??BOLSTEP, BOLSTEP);
            $limit=min($vars['limit']??BOLLIMIT, BOLLIMIT);
            $results=$limit;
            unset($vars['limit']);
            unset($vars['offset']);
            unset($vars['format']);
            unset($vars['apikey']);
            unset($vars['view']);

            if ($parsed['host']=='api.bol.com') {
                $apivars=$vars;
                $apiurl='https://api.bol.com'.$parsed['path'];
            } else {
                if (!preg_match('#(be|nl)/nl/([a-z])/#', $parsed['path'], $m)) {
                         $this->error("UNknown URL Pattern");
                          $this->status=false;
                          $this->items=array();
                           return;
                }
                $country=$m[1];
                $type=$m[2];
                switch ($country) {
                    case 'nl':
                        break;
                    case 'be':
                        $apivars['country']='BE';
                        break;
                    default:
                }
                switch ($type) {
                    case 's':
                        $apiurl="https://api.bol.com/catalog/v4/search/";
                        $search=$vars['searchtext']??'';
                        if (! empty($vars['N'])) {
                            $ids[]=$vars['N'];
                        }

                        if (! empty($vars['filter_N'])) {
                            $ids=array_merge($ids, explode(' ', $vars['filter_N']));
                        }
                        if ($search) {
                            $apivars['q']=$search;
                            $apiurl="https://api.bol.com/catalog/v4/search/";
                        } else {
                            //voor als iemand de url handmatig aangemast heeft en de s
                            $apiurl="https://api.bol.com/catalog/v4/lists/";
                        }

                        break;
                    case 'l':   # inteded fall tru
                        $apiurl="https://api.bol.com/catalog/v4/lists/";
                        if (preg_match_all('#[\+/]([0-9]+)#', $url, $m)) {
                            $ids=array_merge($ids, $m[1]);
                        }
                        break;
                    case 'p':   # inteded fall tru
                        $apiurl="https://api.bol.com/catalog/v4/products/";
                        if (!preg_match('#/([0-9]+)[/?$]#', $url, $m)) {
                            $product=array_merge($ids, $m[1]);
                             $this->error("Product not found ");
                              $this->status=false;
                              $this->items=array();
                            return;
                        }
                        $apiurl .=(int)$m[1];
                        $limit=1;
                        break;
                    default:
                         $this->error("Unknown Page Type: $type");
                          $this->status=false;
                          $this->items=array();
                        return;
                }
            }
            if (! $apiurl) {
                print "Niks gevonden";
                return;
            }
            if ($ids) {
                        $apivars['ids']=join(',', $ids);
            }
        
            $apivars['limit']=$step;
       	    if ( defined('BOLAPIKEY') && !defined('BOLCLIENTID') ) {
               $apivars['apikey']=BOLAPIKEY;
	    }
            $apivars['format']='json';
            $apivars['offers']='newoffers';
            $apivars['includeattributes']=1;
            $apiurl =$apiurl .'?'. http_build_query($apivars);
            while ($start < $results) {
                $urlstep =$apiurl .'&offset='. $start;
                $start+=$step; // if it fails start will be > results
       	        if ( defined('BOLAPIKEY') && !defined('BOLCLIENTID') ) {
                   $urlstep=prefetch($urlstep);
		} else {
                   $urlstep=prefetchBol($urlstep);
		}
                $json=file_get_contents($urlstep);
                if ($json) {
                    $data=json_decode($json);
                    if (isset($data->Message)) {
                        print "$data->Message\n";
                        return;
                    }
                    $results=min($limit, $data->totalResultSize??0);
                    if ($data && is_array($data->products)) {
                        $search=$data->originalRequest->category->name??$search;
                        foreach ($data->products as $k => $result) {
                            foreach ($result->urls as $_url) {
                                $shopurl=$_url->value;
                                if ($_url->key=='DESKTOP') {
                                    break;
                                }
                            }
                            if (!$shopurl) {
                                  continue;
                            }
                            $image=false;
                            if (is_array($result->images??false)) {
                                $image=array_pop($result->images)->url??false;
                            }
                            if (!$image) {
                                  continue;
                            }
                            $price=false;
                            $seller='';
                            if (isset($result->offerData->offers)) {
                                foreach ($result->offerData->offers as $offer) {
                                    if (!$price || ($offer->bestOffer==1)) {
                                        $price=$offer->price;
                                        $seller=$offer->seller->displayName??'';
                                        ;
                                        break;
                                    }
                                }
                            }

                            if (!$price) {
                                  continue;
                            }
                            $attr=[];
                            if (!empty($result->attributeGroups) && is_array($result->attributeGroups)) {
                                foreach ($result->attributeGroups as $group) {
                                    if (!empty($group->attributes) && is_array($group->attributes)) {
                                        foreach ($group->attributes as $attribute) {
                                            $value=strtolower($attribute->value??'');
                                            $label=strtolower($attribute->label??'');
                                            if ($value) {
                                                                $attr[$label] = (isset($attr[$label])?$attr[$label] . ',':'').$attribute->value;
                                            }
                                        }
                                    }
                                }
                                unset($result->attributeGroups);
                            }
                            if (!empty($result->entityGroups) && is_array($result->entityGroups)) {
                                foreach ($result->entityGroups as $group) {
                                    $label=strtolower($group->title??'');
                                    if (!empty($group->entities) && is_array($group->entities)) {
                                        foreach ($group->entities as $attribute) {
                                            $value=strtolower($attribute->value??'');
                                            if ($value) {
                                                                $attr[$label] = (isset($attr[$label])?$attr[$label] . ':':'').$attribute->value;
                                            }
                                        }
                                    }
                                }
                                unset($result->entityGroups);
                            }
                            $cats=[];
                            if (!empty($result->parentCategoryPaths)&& is_array($result->parentCategoryPaths)) {
                                foreach ($result->parentCategoryPaths as $path) {
                                    if (!empty($path->parentCategories) && is_array($path->parentCategories)) {
                                        foreach ($path->parentCategories as $cat) {
                                            if (!empty($cat->name)) {
                                                $cats[]=$cat->name;
                                            }
                                        }
                                    }
                                }
                            }

                            $cats=array_values(array_unique($cats));
                            unset($result->urls);
                            unset($result->images);
                            unset($result->media);
                            unset($result->offerData);
                            unset($result->parentCategoryPaths);
                            $result=$this->_squash($result);
                            $result=array_merge($attr, $result);
                            foreach ($cats as $k => $cat) {
                                $result["parentcategorie_{$k}"]=$cat;
                            }
                            $result['description']=!empty($result['longdescription'])?$result['longdescription']:$result['shortdescription']??$result['summary']??'';
                            $result['search']=$search;

                            $result['durl']=$shopurl;
                            $result['url']=sprintf(
                                "https://partnerprogramma.bol.com/click/click?p=1&t=url&s=%s&url=%s&f=TXL&name=%s&subid=DF-REF",
                                BOLAFFID,
                                urlencode($shopurl),
                                BOLAFFNAME
                            );
                            $result['image']=$image;
                            $result['price']=$price;
                            $result['seller']=$seller;
                            //$result['md5']=md5('bol'.$result['ean']??'');
                            $this->store_item($result);
                        }
                    }
                }
            }
        }
    }
} // end class RSS

if (file_exists(MAGPIE_DIR . 'xml_custom.php')) {
    require_once(MAGPIE_DIR . 'xml_custom.php');
}

$feed_parsers["AutoCSV"]= 'CSV/Autodetect' ;
$feed_parsers["M4NCSV"]= 'CSV tab & "' ;
$feed_parsers["AffiliateCSV"]= 'CSV ; & "' ;
$feed_parsers["WebgainsCSV"]= 'CSV , & "' ;
$feed_parsers["TradedoublerCSV"]= 'CSV | & "' ;
$feed_parsers["DaisyconRSS"]= 'Daisycon' ;
$feed_parsers['TradetrackerV2']= 'TT' ;
$feed_parsers['NetaffiliationRSS']= 'Netaffiliation' ;
$feed_parsers['WebgainsRSS']= 'Webgains' ;
$feed_parsers["ProductserveRSS"]= 'AffiliateWindow' ;
$feed_parsers["TradeDoublerRSS"]= 'TD' ;
$feed_parsers["JsonCSV"]= 'Json' ;
$feed_parsers['BolUrl']= 'Bol via Url' ;

#$feed_parsers['GSRSS']= 'Google shopping' ;
