<?php
/**
 * https://affiliatefeeds.nl
 * replaces the core joomla rss renderer for joomla datafeeds.
 * the core rss does not allow custom feeds ( georss )
 * and does not parse the categories well.
 *
 * Joomla! is free software. This version may have been modified pursuant
 * to the GNU General Public License, and as distributed it includes or
 * is derivative of works licensed under the GNU General Public License or
 * other free or open source software licenses.
 * See COPYRIGHT.php for copyright notices and details.
 */

// Check to ensure this file is within the rest of the framework
defined('JPATH_BASE') or die();

class JDocumentRendererRSS extends JDocumentRenderer
{
    public $_mime = 'application/rss+xml';

    /**
     * Render the feed.
     *
     */
    public function render($name = null, $params = null, $content = null)
    {
        $now = JFactory::getDate();
        $data = &$this->_doc;

        $uri = JFactory::getURI();
        $url = $uri->toString(['scheme', 'user', 'pass', 'host', 'port']);
        $syndicationURL = JRoute::_('&format=feed');

        $feed = "<rss version=\"2.0\" xmlns:atom=\"http://www.w3.org/2005/Atom\" xmlns:georss=\"http://www.georss.org/georss\">\n";
        $feed .= "	<channel>\n";
        $feed .= '               <title><![CDATA['.$data->title."]]></title>\n";

        $feed .= '			<description><![CDATA['.$data->description."]]></description>\n";
        $feed .= '		<link>'.str_replace(' ', '%20', $url.$data->link)."</link>\n";
        $feed .= '		<lastBuildDate>'.htmlspecialchars($now->toRFC822(), ENT_COMPAT, 'UTF-8')."</lastBuildDate>\n";
        $feed .= '		<generator>'.$data->getGenerator()."</generator>\n";
        $feed .= '       <atom:link rel="self" type="application/atom+xml" href="'.str_replace(' ', '%20', $url.$syndicationURL).'" />';

        if ('' != $data->language) {
            $feed .= '		<language>'.$data->language."</language>\n";
        }
        if ('' != $data->pubDate) {
            $pubDate = &JFactory::getDate($data->pubDate);
            $feed .= '		<pubDate>'.htmlspecialchars($pubDate->toRFC822(), ENT_COMPAT, 'UTF-8')."</pubDate>\n";
        }
        if ('' != $data->category) {
            $feed .= '		<category>'.htmlspecialchars($data->category, ENT_COMPAT, 'UTF-8')."</category>\n";
        }
        if ('' != $data->docs) {
            $feed .= '		<docs>'.htmlspecialchars($data->docs, ENT_COMPAT, 'UTF-8')."</docs>\n";
        }

        for ($i = 0; $i < count($data->items); ++$i) {
            if ((false === strpos($data->items[$i]->link, 'http://')) and (false === strpos($data->items[$i]->link, 'https://'))) {
                $data->items[$i]->link = str_replace(' ', '+', $url.$data->items[$i]->link);
            }
            $feed .= "		<item>\n";
            $feed .= '			<title>'.htmlspecialchars(strip_tags($data->items[$i]->title), ENT_COMPAT, 'UTF-8')."</title>\n";
            $feed .= '			<link>'.str_replace(' ', '%20', $data->items[$i]->link)."</link>\n";
            if (preg_match('/(<georss:point>.*?>)/', $data->items[$i]->description, $m)) {
                $feed .= $m[1];
            }

            if (!empty($data->items[$i]->extradata['georss'])) {
                $feed .= '<georss:point>'.$data->items[$i]->extradata['georss'].'</georss:point>';
            }
            $feed .= '			<description><![CDATA['.$this->_relToAbs($data->items[$i]->description)."]]></description>\n";

            if ('' != $data->items[$i]->category) {
                foreach (explode(',', $data->items[$i]->category) as $v) {
                    $v = trim($v);
                    if (strlen($v)) {
                        $feed .= '			<category>'.htmlspecialchars($v, ENT_COMPAT, 'UTF-8')."</category>\n";
                    }
                }
            }
            if ('' != $data->items[$i]->comments) {
                $feed .= '			<comments>'.htmlspecialchars($data->items[$i]->comments, ENT_COMPAT, 'UTF-8')."</comments>\n";
            }
            if ('' != $data->items[$i]->date) {
                $itemDate = &JFactory::getDate($data->items[$i]->date);
                $feed .= '			<pubDate>'.htmlspecialchars($itemDate->toRFC822(), ENT_COMPAT, 'UTF-8')."</pubDate>\n";
            }
            if ('' != $data->items[$i]->guid) {
                $feed .= '			<guid>'.htmlspecialchars($data->items[$i]->guid, ENT_COMPAT, 'UTF-8')."</guid>\n";
            }

            $feed .= "		</item>\n";
        }
        $feed .= "	</channel>\n";
        $feed .= "</rss>\n";

        return $feed;
    }

    /**
     * Convert links in a text from relative to absolute.
     *
     */
    public function _relToAbs($text)
    {
        $base = JURI::base();
        $text = preg_replace('/(href|src)="(?!http|ftp|https)([^"]*)"/', "$1=\"$base\$2\"", $text);

        return $text;
    }
}
